#include <ctype.h>

strncasecmp(s1, s2, n)
char *s1, *s2;
int n;
{
   register int i;

   for (i=0; i<n; i++) {
	if (toupper(*s1) == toupper(*s2)) {
		s1++;
		s2++;
	} else
		return (1);
   }
   return (0);
}
